package org.openxma.addons.ui.table.customizer.xma.client;

import java.util.Locale;
import java.util.ResourceBundle;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import at.spardat.xma.widgets.DatePicker;
import at.spardat.xma.page.*;
import at.spardat.xma.mdl.*;
import at.spardat.enterprise.util.*;
import at.spardat.xma.mdl.list.*;
import at.spardat.xma.mdl.simple.*;
import at.spardat.enterprise.fmt.*;

/**
 * This class implements the generic client part of FilterEmDate.
 * It implements the widgets and models of the page.
 *
 */
public abstract class FilterEmDateGen extends EmbeddedPage {
    private Locale genPageLocale = null;
    private ResourceBundle genPageMessages = null;
    private Scaler scaler = null;
    Composite mainCompositeW;
    Combo operatorComboW;
    IListWMClient operatorCombo;
    Combo linkOperandComboW;
    IListWMClient linkOperandCombo;
    Button addButtonW;
    Label vsep2W;
    Label vsep1W;
    Label vsep3W;
    Button removeButtonW;
    DatePicker valueFilterW;
    ISimpleWMClient valueFilter;
    WModel[] widgetModels;
    Control[] widgets;

    /**
     * Constructs the FilterEmDateGen.
     * @param parent the page where this page is embedded.
     */
    public FilterEmDateGen(PageClient parent) {
        super(parent,false);
        createModels();
    }

    /*
     * returns the Locale used for this page
    */
    final protected Locale getGenPageLocale() {
        if (genPageLocale == null) {
            genPageLocale = getComponent().getContext().getLocale();
        }
        return genPageLocale;
    }

    /*
     * returns the message resource bundle used for this page
    */
    final protected ResourceBundle getGenPageMessages() {
        if (genPageMessages == null) {
            genPageMessages = ResourceBundle.getBundle("org.openxma.addons.ui.table.customizer.xma.client.TableCustomizerComp",getGenPageLocale());
        }
        return genPageMessages;
    }

    /*
     * returns the scaler used for this page
    */
    final protected Scaler getScaler() {
        if (scaler == null) {
            scaler = Scaler.getInstance(getComposite());
        }
        return scaler;
    }

    /**
     * Gives the URI of the help for this page
     * This URI is relative to the download-URI of the component
     * @return the relative URI of the help for this page
     */
    public String getHelpUri() {
        return "help/org/openxma/addons/ui/table/customizer/xma/FilterEm.html";
    }

    /**
     * Create and initialize the WidgetModels of the page.
     * Subpages which are not lazy are created too.
     */
    public void createModels() {
        IFmt formatter;
        operatorCombo = new ListWMClient((short) 0,this, ListWMClient.S_NULL);
        linkOperandCombo = new ListWMClient((short) 1,this, ListWMClient.S_NULL);
        valueFilter = new SimpleWMClient((short) 2, Types.T_DATE, this);
        formatter = ADateFmt.getInstance(ADateFmt.MEDIUM|ADateFmt.MANDATORY, getGenPageLocale());
        valueFilter.setFmt(formatter);

        widgetModels = new WModel[] { (WModel) operatorCombo, (WModel) linkOperandCombo, (WModel) valueFilter };
        getComponent().registerPageModel(this);
    }

    /**
     * Indicate if the WidgetModels are already created.
     * @return true if the WidgetModels exist.
     */
    public boolean hasModels() {
        return widgetModels != null;
    }

    /**
     * Remove all WidgetModels of the page.
     */
    public void removeWidgetModels() {
        operatorCombo = null;
        linkOperandCombo = null;
        valueFilter = null;
        widgetModels = null;
    }

    /**
     * Returns an array of all WidgetModels of this page.
     * @return array of WidgetModels. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public WModel[] getWModels() {
        return widgetModels;
    }

    /**
     * Replaces the model array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWModels(WModel[] models) {
        widgetModels=models;
    }

    /**
     * Returns a unique id to identify the type of this Page.
     * @return the typeId
     */
    public short getTypeId() {
        return TableCustomizerComp.FilterEmDateClass;
    }

    /**
     * Returns the component this page belongs to, casted to its actual type.
     * @return The Component. Never null.
     */
    public TableCustomizerComp getTypedComponent() {
        return (TableCustomizerComp) getComponent();
    }

    /**
     * Returns an array of all Widgets of this page.
     * @return array of Widgets. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public Control[] getWidgets() {
        return widgets;
    }

    /**
     * Replaces the widget array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWidgets(Control[] widgets) {
        this.widgets=widgets;
    }

    /**
     * Create and initialize all SWT-Widgets of the page.
     * It <br>
     * <ul>
     * <li> instantiates the Widgets </li>
     * <li> sets constant texts (Title, Labels, Tooltips) and Images </li>
     * <li> registers the eventadapter </li>
     * <li> defines the Layout (Attachments, Distances, Sizes) </li>
     * <li> sets the focus </li>
     * <li> defines the Taborder </li>
     * </ul>
     */
    protected void createWidgets() {
        EventAdapter adapter = new EventAdapter(this);
        final Scaler scaler = getScaler();

        Composite filterEmCompW = getComposite();
        filterEmCompW.addHelpListener(adapter);
        FormLayout layout = new FormLayout();
        layout.marginHeight = scaler.convertYToCurrent(3);
        layout.marginWidth = scaler.convertXToCurrent(3);
        filterEmCompW.setLayout(layout);

        mainCompositeW = new Composite(filterEmCompW,SWT.NONE);
        layout = new FormLayout();
        layout.marginHeight = scaler.convertYToCurrent(3);
        layout.marginWidth = scaler.convertXToCurrent(3);
        mainCompositeW.setLayout(layout);
        mainCompositeW.setData("WIDGET_ID","mainCompositeW");

        operatorComboW = new Combo(mainCompositeW,SWT.DROP_DOWN|SWT.BORDER|SWT.READ_ONLY);
        operatorComboW.setData("WIDGET_ID","operatorComboW");
        operatorComboW.addFocusListener(adapter);
        operatorComboW.addModifyListener(adapter);
        operatorComboW.addSelectionListener(adapter);

        linkOperandComboW = new Combo(mainCompositeW,SWT.DROP_DOWN|SWT.BORDER);
        linkOperandComboW.setData("WIDGET_ID","linkOperandComboW");
        linkOperandComboW.addFocusListener(adapter);
        linkOperandComboW.addModifyListener(adapter);
        linkOperandComboW.addSelectionListener(adapter);

        addButtonW = new Button(mainCompositeW, SWT.PUSH|SWT.CENTER);
        addButtonW.setText(getGenPageMessages().getString("FilterEmDate.addButtonW"));
        addButtonW.setData("WIDGET_ID","addButtonW");
        addButtonW.addSelectionListener(adapter);

        vsep2W = new Label (mainCompositeW,SWT.SEPARATOR|SWT.VERTICAL);
        vsep2W.setVisible(false);
        vsep2W.setData("WIDGET_ID","vsep2W");

        vsep1W = new Label (mainCompositeW,SWT.SEPARATOR|SWT.VERTICAL);
        vsep1W.setVisible(false);
        vsep1W.setData("WIDGET_ID","vsep1W");

        vsep3W = new Label (mainCompositeW,SWT.SEPARATOR|SWT.VERTICAL);
        vsep3W.setVisible(false);
        vsep3W.setData("WIDGET_ID","vsep3W");

        removeButtonW = new Button(mainCompositeW, SWT.PUSH|SWT.CENTER);
        removeButtonW.setText(getGenPageMessages().getString("FilterEmDate.removeButtonW"));
        removeButtonW.setData("WIDGET_ID","removeButtonW");
        removeButtonW.addSelectionListener(adapter);

        valueFilterW = new DatePicker(mainCompositeW,SWT.SINGLE|SWT.BORDER|SWT.LEFT,DatePicker.RED_WEEKEND);
        valueFilterW.setDatePickerLocale(getGenPageLocale());
        valueFilterW.setData("WIDGET_ID","valueFilterW");
        valueFilterW.getDateTextField().addFocusListener(adapter);
        valueFilterW.getDateTextField().addModifyListener(adapter);
        valueFilterW.getDateTextField().addVerifyListener(adapter);

        widgets = new Control[] { mainCompositeW, operatorComboW, linkOperandComboW, addButtonW, vsep2W, vsep1W, vsep3W, removeButtonW, valueFilterW };

        createWidgetsLayout();
        createWidgetsTabOrder();

    }

    /**
     * Create the layout of all SWT-Widgets of the page.
     * (Attachments, Distances, Sizes)
     */
    public void createWidgetsLayout() {
        FormData data;
        final Scaler scaler = getScaler();
        data = new FormData();
        data.height = scaler.convertYToCurrent(27);
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.right = new FormAttachment(100,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(0,100,scaler.convertYToCurrent(0));
        mainCompositeW.setLayoutData(data);

        data = new FormData();
        data.width = scaler.convertXToCurrent(30);
        data.left = new FormAttachment(linkOperandComboW,scaler.convertXToCurrent(3),SWT.RIGHT);
        data.top = new FormAttachment(valueFilterW,scaler.convertYToCurrent(0),SWT.CENTER);
        operatorComboW.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(vsep1W,scaler.convertXToCurrent(0),SWT.RIGHT);
        data.top = new FormAttachment(valueFilterW,scaler.convertYToCurrent(0),SWT.CENTER);
        linkOperandComboW.setLayoutData(data);

        data = new FormData();
        data.width = scaler.convertXToCurrent(20);
        data.left = new FormAttachment(valueFilterW,scaler.convertXToCurrent(3),SWT.RIGHT);
        data.top = new FormAttachment(valueFilterW,scaler.convertYToCurrent(0),SWT.CENTER);
        addButtonW.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(vsep1W,scaler.convertXToCurrent(70),SWT.RIGHT);
        data.top = new FormAttachment(0,100,scaler.convertYToCurrent(0));
        vsep2W.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(0,100,scaler.convertYToCurrent(0));
        vsep1W.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(vsep2W,scaler.convertXToCurrent(50),SWT.RIGHT);
        data.top = new FormAttachment(0,100,scaler.convertYToCurrent(0));
        vsep3W.setLayoutData(data);

        data = new FormData();
        data.width = scaler.convertXToCurrent(20);
        data.left = new FormAttachment(addButtonW,scaler.convertXToCurrent(3),SWT.RIGHT);
        data.top = new FormAttachment(valueFilterW,scaler.convertYToCurrent(0),SWT.CENTER);
        removeButtonW.setLayoutData(data);

        data = new FormData();
        data.width = scaler.convertXToCurrent(150);
        data.left = new FormAttachment(operatorComboW,scaler.convertXToCurrent(3),SWT.RIGHT);
        data.right = new FormAttachment(100,100,scaler.convertXToCurrent(-50));
        data.top = new FormAttachment(operatorComboW,scaler.convertYToCurrent(0),SWT.CENTER);
        valueFilterW.setLayoutData(data);

    }

    /**
     * Create the tab order of all SWT-Widgets of the page.
     * (sets the focus and defines the tab order)
     */
    protected void createWidgetsTabOrder() {
        Control[] tabOrder;
        final Composite filterEmCompW = getComposite();
        tabOrder = new Control[1];
        tabOrder[0] = mainCompositeW;
        filterEmCompW.setTabList(tabOrder);
        tabOrder = new Control[5];
        tabOrder[0] = operatorComboW;
        tabOrder[1] = linkOperandComboW;
        tabOrder[2] = addButtonW;
        tabOrder[3] = removeButtonW;
        tabOrder[4] = valueFilterW;
        mainCompositeW.setTabList(tabOrder);
    }

    /**
     * Remove all Widgets of the page after they have been disposed.
     */
    public void removeWidgets() {
        mainCompositeW = null;
        operatorComboW = null;
        linkOperandComboW = null;
        addButtonW = null;
        vsep2W = null;
        vsep1W = null;
        vsep3W = null;
        removeButtonW = null;
        valueFilterW = null;
        widgets = null;
    }


    /**
     * Attaches all WidgetModels of the page to there corresponding Widgets.
     */
    public void attachUIImpl() throws AttachmentExceptionClient {
        operatorComboW.setData(operatorCombo.getUIDelegate());
        operatorCombo.getUIDelegate().attachUI(operatorComboW,null);
        linkOperandComboW.setData(linkOperandCombo.getUIDelegate());
        linkOperandCombo.getUIDelegate().attachUI(linkOperandComboW,null);
        valueFilterW.getDateTextField().setData(valueFilter.getUIDelegate());
        valueFilter.getUIDelegate().attachUI(valueFilterW.getDateTextField(),null);
    }

    /**
     * This method will be called every time the FilterEmDate becomes visible.
     */
    protected void enter() {
        super.enter();
    }

    /**
     * Calls the clientevent configured for the widget of the event.
     *
     * @param event the SWT-Event that happened.
     * @param type the type of the Event, as defined by the event type constants in class SWT.
     */
    protected void clientEvent(SelectionEvent event,int type) {
        if (event.widget == addButtonW) {
            addButtonWEvent();
        } else if (event.widget == removeButtonW) {
            removeButtonWEvent();
        }
    }

    /**
     * This method will be called every time the user clicks on addButtonW.
     */
    abstract protected void addButtonWEvent();

    /**
     * This method will be called every time the user clicks on removeButtonW.
     */
    abstract protected void removeButtonWEvent();

}
