package org.openxma.addons.ui.table.customizer.xma.client;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.openxma.addons.ui.table.customizer.common.CommonColumnModel;
import org.openxma.addons.ui.table.customizer.common.CommonDataModel.CodDatatype;
import org.openxma.addons.ui.table.customizer.common.CommonDataModel.FilterOperator;
import org.openxma.addons.ui.table.customizer.common.CommonFilterModel;
import org.openxma.addons.ui.table.customizer.common.ITableCustomizerUISettings;
import org.openxma.addons.ui.table.customizer.common.TableCustomizerUISettingsFactory;

import at.spardat.enterprise.exc.AppException;
import at.spardat.enterprise.fmt.IFmt;
import at.spardat.enterprise.util.Types;
import at.spardat.xma.boot.component.IXMAControl;
import at.spardat.xma.component.ComponentClient;
import at.spardat.xma.datasource.DomainFmt;
import at.spardat.xma.mdl.list.IListWMClient;
import at.spardat.xma.page.PageClient;
import at.spardat.xma.page.Scaler;

/**
 *
 */
public class FilterPage<T extends PageClient> extends FilterPageGen {

    protected List<T> elements = new ArrayList<T>();
    private CommonColumnModel columnModel;
    private int colId = -1;
    private ITableCustomizerUISettings uiSettings = ((TableCustomizerPage) getParent()).getUISettings();
    private String title;
    
    public String getTitle(){
        return title;
    }
    public void setTitle(String title){
        this.title = title;
    }

    protected List<T> getElements() {
        return elements;
    }

    protected IFmt getCurrentFmt() {
        return ((TableCustomizerPage) getParent()).getTypedComponent().getTableWMClient().getColumn(getColId())
                .getFormatter();
    }

    void addElement(T el) {
        elements.add(el);
    }

    // resets all operators to the given value
    protected void adjustFilterLinkOperators(String operator, T filterEm) {
        int indexOf = elements.indexOf(filterEm);
        if (indexOf >= 0 && operator != null) {
            for (T em : getElements()) {
                if (em == filterEm) {
                    continue;
                }
                if (em instanceof FilterEmText) {
                    IListWMClient linkOperandCombo = ((FilterEmText) em).linkOperandCombo;
                    String selected = linkOperandCombo.getSelected();
                    if (selected != null) {
                        linkOperandCombo.select(operator);
                    }
                }
            }
        }
    }

    @Override
    protected void createWidgets() {
        super.createWidgets();
        uiSettings = ((TableCustomizerPage) getParent()).getUISettings();
        if (uiSettings == null) {
            uiSettings = TableCustomizerUISettingsFactory.createUISettings(((PageClient)((ComponentClient) getTypedComponent().getSession().getComponents().next()).getPageModels().next()),getComposite());
        }
        applyUIPref(widgets);
        applyUIPref(getComposite());


        if (uiSettings.getBackground() != null) {
            scrollCompositeinnerCompW.setBackground(uiSettings.getBackground());
            for (Control cntrl : widgets) {
                cntrl.setBackground(uiSettings.getBackground());
            }
        }
        if (uiSettings.getForeground() != null) {
            scrollCompositeinnerCompW.setForeground(uiSettings.getBackground());
            for (Control cntrl : widgets) {
                cntrl.setForeground(uiSettings.getForeground());
            }
        }
        if (uiSettings.getFont() != null) {
            scrollCompositeinnerCompW.setFont(uiSettings.getFont());
            for (Control cntrl : widgets) {
                cntrl.setFont(uiSettings.getFont());
            }
        }
        getShell().setText(title ==null?"Filter" : title);
        getShell().setImage(getTypedComponent().getImage(TableCustomizerComp.IMG_FILTER));    
    }

    private byte type;

    /**
     * Constructs the FilterPage.
     *
     * @param parent
     *            the parent page
     */
    public FilterPage(PageClient parent) {
        super(parent);
    }

    protected void removeFilter(T emFilter) {
        int index = elements.indexOf(emFilter);
        if (index == -1)
            return;

        // if the deleted element has one widget below, attach it to widget
        // former above the deleted element
        if (index + 1 < elements.size()) { // has element below attach it
            // to previous
            T below = elements.get(index + 1);
            final Scaler scaler = Scaler.getInstance(this.getComposite());
            FormData dataBelow = (FormData) ((IXMAControl) below).getComposite().getLayoutData();
            if (index == 0)
                dataBelow.top = new FormAttachment(0, 100, scaler.convertYToCurrent(0));
            else {
                PageClient previous = (PageClient) elements.get(index - 1);
                dataBelow.top = new FormAttachment(previous.getComposite(), scaler.convertYToCurrent(0));
            }
        }
        elements.remove(index);
        this.removeChild((IXMAControl) emFilter);
        redraw(scrollCompositeinnerCompW);
        stateChanged();
        // finally inform parent
        this.stateChanged();
        this.getDialogPage().stateChangedBaseImpl(); // is needed for enable /
        // disable of button

    }

    protected T addFilter() {
        PageClient filterEm = null;
        if (isText()) {
            filterEm = newFilterForString();
        } else if (isDate()) {
            filterEm = newFilterForDate();
        } else if (isBoolean()) {
            filterEm = newFilterForBoolean();
        } else if (isNumber()) {
            filterEm = newFilterForNumber();
        } else if (isDOM()) {
            filterEm = newFilterForDOM();
        }        
        addChild((IXMAControl) filterEm);
        if (elements.size()==0) {
            if (isText()) {
                ((FilterEmText) filterEm).removeButtonW.setEnabled(false);
            } else if (isDate()) {
                ((FilterEmDate)filterEm).removeButtonW.setEnabled(false);
            } else if (isBoolean()) {
                ((FilterEmBoolean)filterEm).removeButtonW.setEnabled(false);
            } else if (isNumber()) {
                ((FilterEmNumber)filterEm).removeButtonW.setEnabled(false);
            } else if (isDOM()) {
                ((FilterEmDOM)filterEm).removeButtonW.setEnabled(false);                
            }        
            //first element: disable remove button
            
        }
        final Scaler scaler = Scaler.getInstance(scrollCompositeinnerCompW);
        FormData data = new FormData();
        data.left = new FormAttachment(0, 100, scaler.convertXToCurrent(0));
        data.right = new FormAttachment(100, 100, scaler.convertXToCurrent(0));
        if (elements.isEmpty())
            data.top = new FormAttachment(0, 100, scaler.convertYToCurrent(0));
        else {
            PageClient previous = (PageClient) getElements().get(elements.size() - 1);
            data.top = new FormAttachment(previous.getComposite(), scaler.convertYToCurrent(0));
        }
        ((IXMAControl) filterEm).getComposite().setLayoutData(data);
        addElement((T) filterEm);
       
        redraw(scrollCompositeinnerCompW);
        return (T) filterEm;
    }

    private FilterEmText newFilterForString() {
        FilterEmText filterEm = new FilterEmText(this);
        Composite comp = filterEm.createComposite(scrollCompositeinnerCompW);
        filterEm.linkOperandCombo.add("UND");
        filterEm.linkOperandCombo.add("ODER");
        filterEm.operatorCombo.add("WIE");
        filterEm.operatorCombo.add("NICHT WIE");
        
        filterEm.operatorCombo.select("WIE");
        applyUIPref(new Control[] { comp });

        return filterEm;
    }

    private FilterEmDate newFilterForDate() {
        FilterEmDate filterEm = new FilterEmDate(this);
        applyUIPref(new Control[] { filterEm.createComposite(scrollCompositeinnerCompW) });
        filterEm.linkOperandCombo.add("UND");
        filterEm.linkOperandCombo.add("ODER");
        filterEm.operatorCombo.add("=");
        filterEm.operatorCombo.add("<");
        filterEm.operatorCombo.add("<=");
        filterEm.operatorCombo.add(">");
        filterEm.operatorCombo.add(">=");
        
        filterEm.operatorCombo.select("=");
        return filterEm;
    }
    
    private FilterEmBoolean newFilterForBoolean() {
        FilterEmBoolean filterEm = new FilterEmBoolean(this);
        applyUIPref(new Control[] { filterEm.createComposite(scrollCompositeinnerCompW) });
        filterEm.linkOperandCombo.add("UND");
        filterEm.linkOperandCombo.add("ODER");
        filterEm.operatorCombo.add("JA");
        filterEm.operatorCombo.add("NEIN");
        
        filterEm.operatorCombo.select("JA");
        return filterEm;
    }

    private FilterEmNumber newFilterForNumber() {
        FilterEmNumber filterEm = new FilterEmNumber(this);
        applyUIPref(new Control[] { filterEm.createComposite(scrollCompositeinnerCompW) });
        filterEm.linkOperandCombo.add("UND");
        filterEm.linkOperandCombo.add("ODER");
        filterEm.operatorCombo.add("=");
        filterEm.operatorCombo.add("<");
        filterEm.operatorCombo.add("<=");
        filterEm.operatorCombo.add(">");
        filterEm.operatorCombo.add(">=");
        
        filterEm.operatorCombo.select("=");
        return filterEm;
    }

    protected void applyUIPref(Control widget) {
        if (uiSettings.getBackground() != null) {
            widget.setBackground(uiSettings.getBackground());
        }
        if (uiSettings.getForeground() != null) {
            widget.setForeground(uiSettings.getForeground());
        }
        if (uiSettings.getFont() != null) {
            widget.setFont(uiSettings.getFont());
        }
    }

    protected void applyUIPref(Control[] widgets) {
        for (Control widget : widgets) {
            applyUIPref(widget);
        }
    }

    private FilterEmDOM newFilterForDOM() {
        FilterEmDOM filterEm = new FilterEmDOM(this);
        applyUIPref(new Control[] { filterEm.createComposite(scrollCompositeinnerCompW) });
        filterEm.linkOperandCombo.add("UND");
        filterEm.linkOperandCombo.add("ODER");
        filterEm.operatorCombo.add("=");
        filterEm.operatorCombo.add("!=");
        // get the domain formatter
        IFmt fmt = getCurrentFmt();
        filterEm.operatorCombo.select("=");
        if (fmt != null && fmt instanceof DomainFmt) {
            DomainFmt domainFmt = (DomainFmt) fmt;
            filterEm.dom.setDataSource(domainFmt.getDomain().getSpecTable());
        }
        return filterEm;
    }

    public static void redraw(Composite composite) {
        if (composite.getParent() instanceof ScrolledComposite) {
            Point size = composite.computeSize(SWT.DEFAULT, SWT.DEFAULT);
            ((ScrolledComposite) composite.getParent()).setMinSize(size);
        }

        if (composite.getParent() != null) {
            composite.getParent().layout();
        }

        composite.redraw();
        composite.layout();
    }

    /**
     * Constructs the FilterPage.
     *
     * @param component
     *            the component this page belongs to
     */
    public FilterPage(ComponentClient component) {
        super(component);
    }

    /**
     * Constructs the FilterPage.
     *
     * @param component
     *            the component this page belongs to
     * @param parentShell
     *            the shell of the calling dialog
     */
    public FilterPage(ComponentClient component, Shell parentShell) {
        super(component, parentShell);
    }

    protected void redraw() {
        redraw(scrollCompositeinnerCompW);
    }

    protected void loadFilterByCommonModel(CommonColumnModel cm) {
        for (T em : getElements()) {
            removeFilter(em);
        }
        redraw(); 
        if (cm.getFilterModel().size()>0){
            isNullable.set(cm.isNullableFilter());
        }
        if (isText()) {
            for (CommonFilterModel cfm : cm.getFilterModel()) {
                FilterEmText filterEm = (FilterEmText) addFilter();
                filterEm.linkOperandCombo.select(cm.isAndFilter() ? "UND" : "ODER");
                filterEm.operatorCombo.select(cfm.getCodFilterOperator().getStringValue());
                filterEm.valueFilter.set(replaceWildCardToUI(cfm.getTxtValue()));

            }
        } else if (isNumber()) {
            for (CommonFilterModel cfm : cm.getFilterModel()) {
                FilterEmNumber filterEm = (FilterEmNumber) addFilter();
                filterEm.linkOperandCombo.select(cm.isAndFilter() ? "UND" : "ODER");
                filterEm.operatorCombo.select(cfm.getCodFilterOperator().getStringValue());
                filterEm.valueFilter
                        .setFmt(getTypedComponent().getTableWMClient().getFormatter(cm.getNumColumnIndex()));
                filterEm.valueFilter.set(new BigDecimal(cfm.getTxtValue()));
            }
        } else if (isDate()) {
            for (CommonFilterModel cfm : cm.getFilterModel()) {
                FilterEmDate filterEm = (FilterEmDate) addFilter();
                filterEm.linkOperandCombo.select(cm.isAndFilter() ? "UND" : "ODER");
                filterEm.operatorCombo.select(cfm.getCodFilterOperator().getStringValue());
                try {
                    Date datDate = new SimpleDateFormat("yyyyMMdd").parse(cfm.getTxtValue());
                    filterEm.valueFilter.set(datDate);
                } catch (ParseException e) {
                    throw new AppException("Date validation failed");
                }

            }
        } else if (isBoolean()) {
            for (CommonFilterModel cfm : cm.getFilterModel()) {
                FilterEmBoolean filterEm = (FilterEmBoolean) addFilter();
                filterEm.linkOperandCombo.select(cm.isAndFilter() ? "UND" : "ODER");
                filterEm.operatorCombo.select(cfm.getCodFilterOperator().getStringValue());
                
            }
        } else if (isDOM()) {
            for (CommonFilterModel cfm : cm.getFilterModel()) {
                FilterEmDOM filterEm = (FilterEmDOM) addFilter();
                filterEm.linkOperandCombo.select(cm.isAndFilter() ? "UND" : "ODER");
                filterEm.operatorCombo.select(cfm.getCodFilterOperator().getStringValue());
                filterEm.dom.set(cfm.getTxtValue());
            }
        }
        redraw();
    }

    /**
     * This method will be called every time the FilterPage becomes visible.
     */
    protected void enter() {
        super.enter();
        if (getColumnModel() != null && getColumnModel().getFilterModel().size() > 0) {
            loadFilterByCommonModel(getColumnModel());
        } else {
            addFilter();
        }
        if (isText()) {
            ((FilterEmText) getElements().get(0)).linkOperandComboW.setEnabled(false);
            ((FilterEmText) getElements().get(0)).linkOperandComboW.setVisible(false);
            setFocus(((FilterEmText) getElements().get(0)).valueFilterW);
        } else if (isNumber()) {
            ((FilterEmNumber) getElements().get(0)).linkOperandComboW.setEnabled(false);
            ((FilterEmNumber) getElements().get(0)).linkOperandComboW.setVisible(false);
            setFocus(((FilterEmNumber) getElements().get(0)).valueFilterW);
        } else if (isDate()) {
            ((FilterEmDate) getElements().get(0)).linkOperandComboW.setEnabled(false);
            ((FilterEmDate) getElements().get(0)).linkOperandComboW.setVisible(false);
            setFocus(((FilterEmDate) getElements().get(0)).valueFilterW);
        } else if (isBoolean()) {
            ((FilterEmBoolean) getElements().get(0)).linkOperandComboW.setEnabled(false);
            ((FilterEmBoolean) getElements().get(0)).linkOperandComboW.setVisible(false);
            setFocus(((FilterEmBoolean) getElements().get(0)).operatorComboW);
        } else if (isDOM()) {
            ((FilterEmDOM) getElements().get(0)).linkOperandComboW.setEnabled(false);
            ((FilterEmDOM) getElements().get(0)).linkOperandComboW.setVisible(false);
            setFocus(((FilterEmDOM) getElements().get(0)).domW);
        }
        redraw(scrollCompositeinnerCompW);
    }

    /**
     * Recalculates the state variables of this page. This method is called every time the content of any WizardModel of
     * the Page may have been changed. <br>
     * This method is called imidiatly befor {@link #stateChanged()}.
     */
    public void determineState() {
        super.determineState();
    }

    /**
     * Enables or disables Widgets according to the current state of the page. This method is called every time the
     * content of any WizardModel of the Page may have been changed. <br>
     * This method is called emediatly after {@link #determineState()}.
     */
    public void stateChanged() {
        // If XMA-DSL is used with expression, some enabling/disabling is done
        // in super
        super.stateChanged();
        // TODO implement stateChanged here!
    }

    /**
     * This method will be called every time the user clicks on applyFilterButtonW.
     */
    protected void applyFilterButtonWEvent() {
        List<CommonFilterModel> filterModel = new ArrayList<CommonFilterModel>();
        if (getElements().size() > 0) {
            if (isNullable.isTrue()){
                getColumnModel().setNullableFilter(Boolean.TRUE);
            } else {
                getColumnModel().setNullableFilter(Boolean.FALSE);                
            }
            for (int i = 0; i < getElements().size(); i++) {
                T em = getElements().get(i);
                CommonFilterModel f = new CommonFilterModel();
                if (isText()) {
                    FilterEmText ft = (FilterEmText) em;
                    if (ft.valueFilter.getEncodedValue().trim().length() > 0) {
                        f.setTxtValue(replaceWildCardToBackend(ft.valueFilter.getEncodedValue()).trim());
                        if ("WIE".equals(ft.operatorCombo.getSelected())) {
                            f.setCodFilterOperator(FilterOperator.like);
                        } else {
                            f.setCodFilterOperator(FilterOperator.notlike);
                        }
                        getColumnModel().setAndFilter("UND".equals(ft.linkOperandCombo.getSelected()));
                        getColumnModel().setCodDataType(CodDatatype.STRING);
                        filterModel.add(f);
                        // getColumnModel().setTxtFormatterPattern(txtFormatterPattern)
                    }
                } else if (isDate()) {
                    FilterEmDate ft = (FilterEmDate) em;
                    if (ft.valueFilter.getEncodedValue().trim().length() > 0) {
                        f.setTxtValue(ft.valueFilter.getEncodedValue().trim());
                        f
                                .setCodFilterOperator(FilterOperator.getFilterOperatorByString(ft.operatorCombo
                                        .getSelected()));
                        getColumnModel().setAndFilter("UND".equals(ft.linkOperandCombo.getSelected()));
                        getColumnModel().setCodDataType(CodDatatype.DATE);
                        filterModel.add(f);
                        // Date datDate = new SimpleDateFormat("yyyyMMdd").parse(ft.valueFilter.getEncodedValue());
                    }
                } else if (isBoolean()) {
                    FilterEmBoolean ft = (FilterEmBoolean) em;
                    f.setTxtValue("");
                    f
                    .setCodFilterOperator(FilterOperator.getFilterOperatorByString(ft.operatorCombo
                            .getSelected()));
                    getColumnModel().setAndFilter("UND".equals(ft.linkOperandCombo.getSelected()));
                    getColumnModel().setCodDataType(CodDatatype.BOOL);
                    filterModel.add(f);
                } else if (isNumber()) {
                    FilterEmNumber ft = (FilterEmNumber) em;
                    if (ft.valueFilter.getEncodedValue().trim().length() > 0) {
                        f.setTxtValue(ft.valueFilter.getEncodedValue().trim());
                        f
                                .setCodFilterOperator(FilterOperator.getFilterOperatorByString(ft.operatorCombo
                                        .getSelected()));
                        getColumnModel().setAndFilter("UND".equals(ft.linkOperandCombo.getSelected()));
                        filterModel.add(f);
                    }
                } else if (isDOM()) {
                    FilterEmDOM ft = (FilterEmDOM) em;
                    if (ft.dom.getSelected() != null && ft.dom.getSelected().length() > 0) {
                        f.setTxtValue(ft.dom.getSelected()); // no trim since value must exactly match the dom value
                        f
                                .setCodFilterOperator(FilterOperator.getFilterOperatorByString(ft.operatorCombo
                                        .getSelected()));
                        getColumnModel().setAndFilter("UND".equals(ft.linkOperandCombo.getSelected()));
                        getColumnModel().setCodDataType(CodDatatype.DOM);
                        filterModel.add(f);
                    }
                }
            }
        }
        getColumnModel().setFilterModel(filterModel);
        closeOK();
    }

    /**
     * This method will be called every time the user clicks on cancelButtonW.
     */
    protected void cancelButtonWEvent() {
        closeCancel();
    }


    public void setType(byte type) {
        this.type = type;
    }

    public byte getType() {
        return type;
    }

    boolean isText() {
        return getType() == Types.T_STRING;
    }

    boolean isNumber() {
        return getType() == Types.T_BCD;
    }

    boolean isDate() {
        return getType() == Types.T_DATE;
    }
    
    boolean isBoolean() {
        return getType() == Types.T_BOOLEAN;
    }

    boolean isDOM() {
        return getType() == Types.T_DOM;
    }

    public void setColId(int colId) {
        this.colId = colId;
    }

    public int getColId() {
        return colId;
    }

    public void setColumnModel(CommonColumnModel columnModel) {
        this.columnModel = columnModel;
    }

    public CommonColumnModel getColumnModel() {
        return columnModel;
    }

    private String replaceWildCardToUI(String value) {
        if (value != null) {
            return value.replaceAll("%", "*");
        }
        return value;
    }

    private String replaceWildCardToBackend(String value) {
        if (value != null) {
            return value.replaceAll("\\*", "%");
        }
        return value;
    }

}
