package org.openxma.addons.ui.table.customizer.xma.client;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.openxma.addons.ui.table.customizer.common.CommonColumnModel;
import org.openxma.addons.ui.table.customizer.common.CommonFilterModel;
import org.openxma.addons.ui.table.customizer.common.CommonTableModel;
import org.openxma.addons.ui.table.customizer.common.ITableCustomizerUISettings;
import org.openxma.addons.ui.table.customizer.common.TableCustomizerUISettingsFactory;

import at.spardat.enterprise.exc.AppException;
import at.spardat.enterprise.util.Types;
import at.spardat.xma.component.ComponentClient;
import at.spardat.xma.mdl.table.ITableWMClient;
import at.spardat.xma.mdl.table.TableRow;
import at.spardat.xma.mdl.table.TableUIDelegateClient;
import at.spardat.xma.page.PageClient;
import at.spardat.xma.rpc.RemoteCallClient;
import at.spardat.xma.rpc.RemoteReply;

/**
 *
 */
public class TableCustomizerPage extends TableCustomizerPageGen {

    protected CommonTableModel tableModel;
    String fqTableName;

    

    private ITableCustomizerUISettings uiSettings;




    /**
     * Constructs the TableCustomizerPage.
     *
     * @param parent
     *            the parent page
     */
    public TableCustomizerPage(PageClient parent) {
        super(parent);
    }

    /**
     * Constructs the TableCustomizerPage.
     *
     * @param component
     *            the component this page belongs to
     */
    public TableCustomizerPage(ComponentClient component) {
        super(component);
    }

    /**
     * Constructs the TableCustomizerPage.
     *
     * @param component
     *            the component this page belongs to
     * @param parentShell
     *            the shell of the calling dialog
     */
    public TableCustomizerPage(ComponentClient component, Shell parentShell) {
        super(component, parentShell); 
    }
    
    @Override
    public Composite createComposite(Composite parentComp) {
        if(parentComp instanceof Shell) {
            composite = parentComp;
        } else {
            composite = new Composite(parentComp, SWT.NONE);
        }
        composite.setData(this);
        return composite;
    }

    @Override
    protected void createWidgets() {
        super.createWidgets();

       uiSettings = TableCustomizerUISettingsFactory.createUISettings(((PageClient)((ComponentClient) getTypedComponent().getSession().getComponents().next()).getPageModels().next()),getComposite());

        final Shell shell = getShell();
        String title = uiSettings.getTitle();
        if (title!= null) {
            shell.setText(title);
        }        
        Color background = uiSettings.getBackground();
        if (background!= null) {
            getComposite().setBackground(background);
            sortGroupW.setBackground(background);
            uiGroupW.setBackground(background);
            availableColumnsL.setBackground(background);
            visibleColumnsL.setBackground(background);
            sortCandidateLabel.setBackground(background);
            sortColumnsLabel.setBackground(background);
            upButtonW.setBackground(background);
            downButtonW.setBackground(background);
            sortDownButtonW.setBackground(background);
            sortUpButtonW.setBackground(background);
            setFilterButtonW.setBackground(background);
            removeFilterButtonW.setBackground(background);
            applyAndPersistButtonW.setBackground(background);
            applyButtonW.setBackground(background);
            cancelButtonW.setBackground(background);

        }
        Color foreground = uiSettings.getForeground();
        if (foreground!= null){
            getComposite().setForeground(foreground);
            sortGroupW.setForeground(foreground);
            uiGroupW.setForeground(foreground);
            availableColumnsL.setForeground(foreground);
            visibleColumnsL.setForeground(foreground);
            sortCandidateLabel.setForeground(foreground);
            sortColumnsLabel.setForeground(foreground);
            upButtonW.setForeground(foreground);
            downButtonW.setForeground(foreground);
            sortDownButtonW.setForeground(foreground);
            sortUpButtonW.setForeground(foreground);
            setFilterButtonW.setForeground(foreground);
            removeFilterButtonW.setForeground(foreground);
            applyAndPersistButtonW.setForeground(foreground);
            applyButtonW.setForeground(foreground);
            cancelButtonW.setForeground(foreground);
        }
        if (uiSettings.getFont()!= null) {
            getComposite().setFont(uiSettings.getFont());
        }
        if (uiSettings.getTitle()!= null) {
            getShell().setText(uiSettings.getTitle());
        }
        if (uiSettings.getCustomizerImage()!= null) {
            getShell().setImage(uiSettings.getCustomizerImage());
        }
    }


    /**
     * This method will be called every time the TableCustomizerPage becomes
     * visible.
     */
    protected void enter() {
        super.enter();
        fqTableName = ((ITableCustomizer) getTypedComponent()).getFQTableName();

        RemoteCallClient call = newRemoteCall("onInit");
        RemoteReply reply = call.execute();
        if (reply.getParameter(0) == null) {
            // fill the common model and senbd it to server
            tableModel = getTypedComponent().commonModelFromTable(fqTableName);
            pushTableModel(tableModel);
        } else {// persisted model was found => reload GUI elements
            tableModel = (CommonTableModel) reply.getParameter(0);
        }
        commonModel2GUI();
    }

    /**
     * pushes the current settings to the server without persisting them
     *
     */
    private void pushTableModel(CommonTableModel tableModel) {
        RemoteCallClient push = newRemoteCall("onPushTableModel");
        push.setParameter(0, tableModel);
        push.execute();
    }

    private void commonModel2GUI() {
        availableColumns.clear();
        visibleColumnsTable.clear();
        for (CommonColumnModel cm : tableModel.getColumnModels()) {
            if (cm.isVisible()) {
                new TableRow(visibleColumnsTable, cm.getNamColumn(), new Object[]{"", cm.getTxtColumn()}, cm.getFilterModel().size()>0?TableCustomizerComp.IMG_FILTER:0);
            } else {
                availableColumns.add(cm.getTxtColumn());
            }
        }
        String sortSelection = sortingTable.getSelected();
        refreshSortingGroup();
        if (sortSelection != null) {
            sortingTable.select(sortSelection);
        }
    }

    private void refreshSortingGroup() {
        sortingTable.clear();
        sortingCandidatesTable.clear();
        List<CommonColumnModel> sortableColumnsSortedByNumSort = tableModel.getSortableColumnsSortedByNumSort();
        for (CommonColumnModel cm : sortableColumnsSortedByNumSort ) {
            if (cm.isAscending()) {
                new TableRow(sortingTable, cm.getNamColumn(), new Object[]{"",cm.getTxtColumn()}, TableCustomizerComp.IMG_SORT_ASC);
            } else {
                new TableRow(sortingTable, cm.getNamColumn(), new Object[]{"",cm.getTxtColumn()}, TableCustomizerComp.IMG_SORT_DESC);
            }
        }
        for (CommonColumnModel cm : tableModel.getNotSortableColumnsSortedByNumOrder()) {
            new TableRow(sortingCandidatesTable, cm.getNamColumn(), new Object[]{cm.getTxtColumn()}, 0);
        }
    }



    @Override
    public void initGUI() {
        super.initGUI();
    }


    /**
     * Recalculates the state variables of this page. This method is called
     * every time the content of any WizardModel of the Page may have been
     * changed. <br>
     * This method is called imidiatly befor {@link #stateChanged()}.
     */
    public void determineState() {
        super.determineState();
    }

    /**
     * Enables or disables Widgets according to the current state of the page.
     * This method is called every time the content of any WizardModel of the
     * Page may have been changed. <br>
     * This method is called emediatly after {@link #determineState()}.
     */
    public void stateChanged() {
        // If XMA-DSL is used with expression, some enabling/disabling is done
        // in super
        super.stateChanged();
        addButtonW.setEnabled(availableColumns.getSelected() != null
                && tableModel!= null && !visibleColumnsTable.containsKey(tableModel.getColumnModelByTxtColumn(availableColumns.getSelected()).getNamColumn()));
        removeButtonW.setEnabled(visibleColumnsTable.getSelected() != null);
        setFilterButtonW.setEnabled(visibleColumnsTable.getSelected()!=null);
        removeButtonW.setEnabled(visibleColumnsTable.getSelected()!=null);

        String sortSelected = sortingTable.getSelected();
        String sortCandidateSelected = sortingCandidatesTable.getSelected();
        if (tableModel!= null) {
            TableRow row = sortingTable.getRow(sortSelected);
            boolean isTop = row != null && row.getUIRowIndex()==0;
            boolean isBottom = row != null && row.getUIRowIndex()==sortingTable.size()-1;
            CommonColumnModel columnModelByName = tableModel.getColumnModelByName(sortSelected);
            boolean ascDesc = columnModelByName != null && (columnModelByName.isAscending() || columnModelByName.isDescending());
            sortUpButtonW.setEnabled(ascDesc && !isTop);
            sortDownButtonW.setEnabled(ascDesc && !isBottom);
            addSortButtonW.setEnabled(sortCandidateSelected != null);
            removeSortButtonW.setEnabled(sortSelected != null);
        }




    }

    /**
     * This method will be called every time the user clicks on addButtonW.
     */
    protected void addButtonWEvent() {
        String selected = availableColumns.getSelected();
        tableModel.show(selected);
        refreshAvailableColumnsList();
        refreshVisibleColumns();
        refreshSortingGroup();
    }

    private void refreshVisibleColumns() {
        visibleColumnsTable.clear();
        for (CommonColumnModel cm : tableModel.getColumnModels()) {
            if (cm.isVisible()) {
                new TableRow(visibleColumnsTable, cm.getNamColumn(), new Object[]{"", cm.getTxtColumn()}, cm.getFilterModel().size()>0?TableCustomizerComp.IMG_FILTER:0);
            }
        }
    }

    /**
     * This method will be called every time the user clicks on removeButtonW.
     */
    protected void removeButtonWEvent() {
        String selected = visibleColumnsTable.getSelected();
        tableModel.hide(tableModel.getColumnModelByName(selected).getTxtColumn());
        refreshVisibleColumns();
        visibleColumnsTable.deselectAll();
        refreshAvailableColumnsList();
        refreshSortingGroup();
    }

    private void refreshAvailableColumnsList() {
        availableColumns.clear();
        List<String> availableTxtColumnsAlphabetically = tableModel.getAvailableTxtColumnsAlphabetically();
        availableColumns.add(availableTxtColumnsAlphabetically);
    }

    /**
     * This method will be called every time the user clicks on upButtonW.
     */
    protected void upButtonWEvent() {
        String selected = visibleColumnsTable.getSelected();
        if (selected != null) {
            String txtColumn = tableModel.getColumnModelByName(selected).getTxtColumn();
            tableModel.moveUp(txtColumn);
            refreshVisibleColumns();
            visibleColumnsTable.select(selected);
        }
    }

    /**
     * This method will be called every time the user clicks on downButtonW.
     */
    protected void downButtonWEvent() {
        String selected = visibleColumnsTable.getSelected();
        if (selected != null) {
            String txtColumn = tableModel.getColumnModelByName(selected).getTxtColumn();
            tableModel.moveDown(txtColumn);
            refreshVisibleColumns();
            visibleColumnsTable.select(selected);
        }
    }

    /**
     * This method will be called every time the user clicks on sortAscButtonW.
     */
    protected void sortUpButtonWEvent() {
        String selected = sortingTable.getSelected();
        if (selected != null) {
            tableModel.sortUp(selected);
            refreshSortingGroup();
            sortingTable.select(selected);
        }
    }

    /**
     * This method will be called every time the user clicks on sortDescButtonW.
     */
    protected void sortDownButtonWEvent() {
        String selected = sortingTable.getSelected();
        if (selected != null) {
            tableModel.sortDown(selected);
            refreshSortingGroup();
            sortingTable.select(selected);
        }
    }

    /**
     * This method will be called every time the user clicks on
     * setFilterButtonW.
     */
    protected void setFilterButtonWEvent() {
        String selected = visibleColumnsTable.getSelected();
        if (selected != null) {           
            callFilterDialog(selected);            
        }
    }

    public FilterPage callFilterDialog(String selected) {
        if (tableModel==null) {
            enter();
        }
        CommonColumnModel columnModelByName = tableModel.getColumnModelByName(selected);
        Integer numColumnIndex = columnModelByName.getNumColumnIndex();
        byte columnType = getTableWMClient().getColumnType(numColumnIndex);
        FilterPage fp = null;
        switch (columnType) {
        case Types.T_STRING:
            fp = new FilterPage<FilterEmText>(this);
            break;
        case Types.T_DATE:
            fp = new FilterPage<FilterEmDate>(this);
            break;
        case Types.T_BOOLEAN:
            fp = new FilterPage<FilterEmBoolean>(this);
            break;
        case Types.T_BCD:
            fp = new FilterPage<FilterEmNumber>(this);
            break;
        case Types.T_DOM:
            fp = new FilterPage<FilterEmDOM>(this);
            break;
        default:
            throw new AppException("Unsupported type: " + columnType);
        }
        fp.setType(columnType);
        fp.setColId(numColumnIndex);
        fp.setColumnModel(columnModelByName);     
        
        fp.setTitle("Filter \"" +  columnModelByName.getTxtColumn() + "\"");
        try {
            if (fp.invoke()) { 
                List<CommonFilterModel> filterModel = fp.getColumnModel().getFilterModel();                    
                tableModel.getColumnModelByName(selected).setFilterModel(filterModel);
                refreshVisibleColumns();
                System.out.println("added filter: " + filterModel);
            } else {
                System.out.println("cancel filter settings for " + selected);
            }
        } finally {
            // no dispose method on page
        }
        return fp;
    }

    /**
     * This method will be called every time the user clicks on
     * removeFilterButtonW.
     */
    protected void removeFilterButtonWEvent() {
        String selected = visibleColumnsTable.getSelected();
        if (selected!= null) {
            CommonColumnModel columnModelByName = tableModel.getColumnModelByName(selected);
            columnModelByName.setFilterModel(null);
            commonModel2GUI();
            visibleColumnsTable.select(selected);
        }
    }

    @Override
    protected void cancelButtonWEvent() {
        closeCancel();
    }


    private Table getTable() {
        return (Table) (getTableUIDeligate()).getUIControl();
    }

    private at.spardat.xma.mdl.table.TableUIDelegateClient getTableUIDeligate() {
        return (TableUIDelegateClient) getTableWMClient().getUIDelegate();
    }

    private ITableWMClient getTableWMClient() {
        return getTypedComponent().getTableWMClient();
    }



    protected void applyAndPersistButtonWEvent() {
        applyChangesOnGUI();
        RemoteCallClient call = newRemoteCall("onPersist");
        call.setParameter(0, tableModel);
        call.execute();
        getTypedComponent().getPagingWMClient().setResultSize(Integer.MAX_VALUE);        
        getTableWMClient().clear();
        getTypedComponent().getPagingWMClient().reload();
        closeOK();
    }


    /**
     * apply the current common user model to the table
     */
    @Override
    protected void applyButtonWEvent() {
        applyOnSession();
        closeOK();

    }

    protected void applyOnSession() {
        applyChangesOnGUI();
        handleSort();        
        pushTableModel(tableModel);
        getTypedComponent().getPagingWMClient().setResultSize(Integer.MAX_VALUE);        
        getTableWMClient().clear();
        getTypedComponent().getPagingWMClient().reload();
    }

    private void applyChangesOnGUI() {
        getTypedComponent().updateTableModelColumnSizesFromUI(tableModel);
        getTypedComponent().applyTableModelOnUI(tableModel);
    }

    private void handleSort() {
        List<CommonColumnModel> sortableColumnsSortedByNumSort = tableModel.getSortableColumnsSortedByNumSort();
        if (sortableColumnsSortedByNumSort.size()>0) {
            CommonColumnModel commonColumnModel = sortableColumnsSortedByNumSort.get(0);
            getTypedComponent().getPagingWMClient().setSortingColumn((short) commonColumnModel.getNumColumnIndex().intValue());
            getTableWMClient().setExternalSorter(getTypedComponent().getPagingWMClient());
            if (commonColumnModel.isAscending()) {
                getTypedComponent().getPagingWMClient().setAscending(true);                
                getTableWMClient().sortNatural();
            } else {
                getTypedComponent().getPagingWMClient().setAscending(false);                
            }
        }
    }




    @Override
    protected void sortingTableWSelected() {


    }

    @Override
    protected void addSortButtonWEvent() {
        String selected = sortingCandidatesTable.getSelected();
        if (selected != null) {
            CommonColumnModel columnModelByName = tableModel.getColumnModelByName(selected);
            tableModel.setSortable(columnModelByName);
            refreshSortingGroup();
        }
    }

    @Override
    protected void removeSortButtonWEvent() {
        String selected = sortingTable.getSelected();
        if (selected != null) {
            CommonColumnModel cm = tableModel.getColumnModelByName(selected);
            cm.setAscending(null);
            cm.setSortOrder(Integer.MAX_VALUE);
            refreshSortingGroup();
        }
    }

    @Override
    protected void sortingCandidatesTableWSelected() {
        // TODO Auto-generated method stub

    }

    @Override
    protected void sortingTableWDoubleClick() {
        String selected = sortingTable.getSelected();
        if (selected != null) {
            // toggle the sorting direction
            CommonColumnModel columnModelByName = tableModel.getColumnModelByName(selected);
            if (columnModelByName.isAscending()){
                columnModelByName.setAscending(Boolean.FALSE);
            } else if (columnModelByName.isDescending()){
                columnModelByName.setAscending(Boolean.TRUE);
            }
            refreshSortingGroup();
            sortingTable.select(selected);
        }

    }

    protected ITableCustomizerUISettings getUISettings(){
        return uiSettings;
    }





}
